"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Api = exports.ApiError = void 0;
const axios_1 = require("axios");
const debug = require("debug");
const FormData = require("form-data");
const fs_1 = require("fs");
class ApiError extends Error {
}
exports.ApiError = ApiError;
class Api {
    constructor(token) {
        this.token = token;
        this.log = debug("tg-io:api");
        this.baseUrl = "https://api.telegram.org/bot";
    }
    /**
     * Call method with params as a formdata
     */
    async upload(method, params) {
        var _a, _b;
        if (!(params instanceof FormData)) {
            let formdata = new FormData();
            for (let [key, value] of Object.entries(params)) {
                let filename = null;
                if (Buffer.isBuffer(value))
                    filename = "file.data";
                else if (fs_1.ReadStream.isReadable(value))
                    filename = (_b = (_a = value === null || value === void 0 ? void 0 : value.path) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : "file.data";
                formdata.append(key, value, filename);
            }
            params = formdata;
        }
        return this.callMethod(method, params);
    }
    async callMethod(method, params) {
        var _a;
        let response;
        try {
            response = await axios_1.default.post(`${this.baseUrl}${this.token}/${method}`, params);
        }
        catch (reqError) {
            if (reqError instanceof axios_1.AxiosError) {
                let errorData = (_a = reqError === null || reqError === void 0 ? void 0 : reqError.response) === null || _a === void 0 ? void 0 : _a.data;
                if (errorData == null)
                    throw reqError;
                let error = new ApiError(errorData.description);
                error.code = errorData.error_code;
                error.method = method;
                error.params = JSON.stringify(params);
                throw error;
            }
            throw reqError;
        }
        finally {
            this.log(`${method} ${response ? "OK" : "ERROR"}`);
        }
        return response.data;
    }
}
exports.Api = Api;
