import { IInlineKeyboardButton } from "./types/IKeyboard";
import { IMessageSendParams } from "./types/params/ISendParams";
declare abstract class BaseButton {
    readonly text: string;
    constructor(text: string);
    abstract toObject(): Record<string, any>;
}
declare abstract class BaseKeyboardBuilder<TButton extends BaseButton> {
    protected buttons: TButton[][];
    add(btn: TButton): this;
    wrap(): this;
    clear(): void;
    abstract createButton(...args: any[]): this;
    abstract build(): Pick<IMessageSendParams, "reply_markup">;
}
export declare class InlineButton extends BaseButton {
    readonly url: string;
    readonly payload: string;
    readonly switchQuery: string;
    readonly switchCurrentChatQuery: string;
    constructor(params: {
        text: string;
        url?: string;
        payload?: string;
        switchQuery?: string;
        switchCurrentChatQuery?: string;
    });
    toObject(): IInlineKeyboardButton;
}
export declare class InlineKeyboardBuilder extends BaseKeyboardBuilder<InlineButton> {
    createButton(...args: ConstructorParameters<typeof InlineButton>): this;
    build(): {
        reply_markup: {
            inline_keyboard: IInlineKeyboardButton[][];
        };
    };
}
export declare class Button extends BaseButton {
    private readonly requestType?;
    constructor(text: string, requestType?: "poll_quiz" | "poll_regular" | "location" | "contact");
    toObject(): {
        text: string;
    };
}
export declare class KeyboardBuilder extends BaseKeyboardBuilder<Button> {
    private additionalOptions;
    setResize(state?: boolean): this;
    setOneTime(state?: boolean): this;
    setSelective(state?: boolean): this;
    setPlaceholder(text: string): this;
    createButton(...args: ConstructorParameters<typeof Button>): this;
    build(): {
        reply_markup: {
            text?: string;
            request_contact?: boolean;
            request_location?: boolean;
            request_poll?: {
                type: "regular" | "quiz";
            };
            keyboard: {
                text: string;
            }[][];
        };
    };
    static remove(remove?: boolean, selective?: boolean): string;
}
export {};
