"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyboardBuilder = exports.Button = exports.InlineKeyboardBuilder = exports.InlineButton = void 0;
class BaseButton {
    constructor(text) {
        this.text = text;
    }
}
class BaseKeyboardBuilder {
    constructor() {
        this.buttons = [];
    }
    add(btn) {
        let lastRow = this.buttons.length - 1;
        if (!this.buttons[lastRow] || this.buttons[lastRow].length >= 8) {
            this.wrap();
            lastRow++;
        }
        this.buttons[lastRow].push(btn);
        return this;
    }
    wrap() {
        this.buttons.push([]);
        return this;
    }
    clear() {
        this.buttons = [];
    }
}
class InlineButton extends BaseButton {
    constructor(params) {
        super(params.text);
        for (let [key, val] of Object.entries(params)) {
            this[key] = val;
        }
    }
    toObject() {
        let btn = {
            text: this.text,
            callback_data: this.payload,
            url: this.url,
            switch_inline_query: this.switchQuery,
            switch_inline_query_current_chat: this.switchCurrentChatQuery
        };
        return Object.entries(btn)
            .filter(([_, v]) => v != null)
            .reduce((acc, [key, val]) => (Object.assign({ [key]: val }, acc)), {});
    }
}
exports.InlineButton = InlineButton;
class InlineKeyboardBuilder extends BaseKeyboardBuilder {
    createButton(...args) {
        return this.add(new InlineButton(...args));
    }
    build() {
        return {
            reply_markup: {
                inline_keyboard: this.buttons.map(row => row.map(btn => btn.toObject()))
            }
        };
    }
}
exports.InlineKeyboardBuilder = InlineKeyboardBuilder;
class Button extends BaseButton {
    constructor(text, requestType) {
        super(text);
        this.requestType = requestType;
    }
    toObject() {
        let btn = {
            text: this.text
        };
        let type = this.requestType;
        if (type === "contact")
            btn["request_contact"] = true;
        else if (type === "location")
            btn["request_location"] = true;
        else if (type === "poll_regular")
            btn["request_poll"] = { type: "regular" };
        else if (type === "poll_quiz")
            btn["request_poll"] = { type: "quiz" };
        return btn;
    }
}
exports.Button = Button;
class KeyboardBuilder extends BaseKeyboardBuilder {
    constructor() {
        super(...arguments);
        this.additionalOptions = {};
    }
    setResize(state = true) {
        this.additionalOptions["resize_keyboard"] = state;
        return this;
    }
    setOneTime(state = true) {
        this.additionalOptions["one_time_keyboard"] = state;
        return this;
    }
    setSelective(state = true) {
        this.additionalOptions["selective"] = state;
        return this;
    }
    setPlaceholder(text) {
        this.additionalOptions["input_field_placeholder"] = text.slice(0, 63);
        return this;
    }
    createButton(...args) {
        return this.add(new Button(...args));
    }
    build() {
        return {
            reply_markup: Object.assign({ keyboard: this.buttons.map(row => row.map(btn => btn.toObject())) }, this.additionalOptions)
        };
    }
    static remove(remove = true, selective = false) {
        return JSON.stringify({
            remove_keyboard: remove,
            selective
        });
    }
}
exports.KeyboardBuilder = KeyboardBuilder;
