"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Middleware = exports.MiddlewareToken = void 0;
class MiddlewareToken {
    constructor() {
        this.passingState = true;
    }
    reset() {
        return this.next();
    }
    get() {
        return this.passingState;
    }
    next() {
        this.passingState = true;
    }
    async complete(fn) {
        if (!this.passingState)
            return;
        this.passingState = false;
        await fn();
    }
}
exports.MiddlewareToken = MiddlewareToken;
class Middleware {
    constructor(token) {
        this.token = token;
        this.handlers = [];
    }
    async handle(arg) {
        let value = arg;
        let next = (a) => {
            if (a)
                value = a;
            return this.token.next();
        };
        for (let handler of this.handlers)
            await this.token.complete(() => handler(value, next));
    }
    add(handler) {
        this.handlers.push(handler);
        return this;
    }
}
exports.Middleware = Middleware;
