"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateHandler = void 0;
const CallbackQueryContext_1 = require("./contexts/CallbackQueryContext");
const MessageContext_1 = require("./contexts/MessageContext");
const Middleware_1 = require("./Middleware");
class UpdateHandler {
    constructor(api) {
        this.api = api;
        this.contextCtorStorage = {
            message: MessageContext_1.MessageContext,
            callback_query: CallbackQueryContext_1.CallbackQueryContext
        };
        this.middlewareToken = new Middleware_1.MiddlewareToken();
        this.baseHandler = new Middleware_1.Middleware(this.middlewareToken);
        this.updates = {};
    }
    async handle(update) {
        this.middlewareToken.reset();
        await this.baseHandler.handle(update);
        for (let updateMiddleware of Object.keys(this.updates)
            .filter(key => Object.keys(update).includes(key))
            .map(key => this.updates[key])) {
            await updateMiddleware.handle(update);
        }
    }
    setContext(event, ctor) {
        this.contextCtorStorage[event] = ctor;
    }
    use(handler) {
        this.baseHandler.add(handler);
    }
    onUpdate(updateKind, handler, skipConvertingToContext = false) {
        if (!this.updates.hasOwnProperty(updateKind)) {
            this.updates[updateKind] = new Middleware_1.Middleware(this.middlewareToken);
            this.updates[updateKind].add((upd, next) => {
                if (!upd.hasOwnProperty(updateKind))
                    return next();
                let evt = skipConvertingToContext || !this.contextCtorStorage.hasOwnProperty(updateKind)
                    ? upd[updateKind]
                    : new this.contextCtorStorage[updateKind](this.api, upd[updateKind]);
                next(evt);
            });
        }
        this.updates[updateKind].add(handler);
    }
    hearCallbackQuery(match, handler) {
        this.onUpdate("callback_query", this.hearEvent(match, handler, "data"));
    }
    hearCommand(match, handler) {
        this.onUpdate("message", this.hearEvent(match, handler, "text"));
    }
    hearEvent(match, handler, matchSouce) {
        return (ctx, next) => {
            let content = ctx === null || ctx === void 0 ? void 0 : ctx[matchSouce];
            let isMatched = ((typeof match === "string" && content === match) ||
                (Array.isArray(match) && match.some(t => t === content)) ||
                (match instanceof RegExp && match.test(content)));
            if (isMatched) {
                ctx.match = match instanceof RegExp
                    ? content.match(match)
                    : [];
                return handler(ctx, next);
            }
            return next();
        };
    }
    onMessageEvent(event, handler) {
        this.onUpdate("message", (upd, next) => event.hasOwnProperty(event) || upd.get(event)
            ? handler(upd, next)
            : next());
    }
}
exports.UpdateHandler = UpdateHandler;
