"use strict";
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Webhook = void 0;
const debug = require("debug");
const https = require("https");
const EventTransport_1 = require("./EventTransport");
const promises_1 = require("fs/promises");
class Webhook extends EventTransport_1.EventTransport {
    constructor(api, options) {
        super(api);
        this.options = options;
        this.log = debug("tg-io:webhook");
    }
    async start(handler, port = 8443) {
        if (this.state === EventTransport_1.EventTransportState.Working)
            return;
        if (!(await this.isValid()))
            this.setupWebhook();
        this.server = await this.createServer(handler);
        this.server.listen(port);
        this.log(`started on ${port} port`);
        this.state = EventTransport_1.EventTransportState.Working;
    }
    stop() {
        if (this.state !== EventTransport_1.EventTransportState.Working)
            return;
        this.log("stopped");
        this.revokeWebhook();
        this.state = EventTransport_1.EventTransportState.Stopped;
    }
    async isValid() {
        let opts = this.options;
        let response = await this.api.callMethod("getWebhookInfo", {});
        return (opts.url === response.url &&
            opts.maxConnections === response.max_connections &&
            opts.ip === response.ip_address);
    }
    setupWebhook() {
        var _a;
        let opts = this.options;
        let params = {
            url: opts.url,
            max_connections: (_a = opts === null || opts === void 0 ? void 0 : opts.maxConnections) !== null && _a !== void 0 ? _a : 40,
            certificate: opts.tls.cert
        };
        if (opts.secret)
            params.secret = opts.secret;
        this.log("configured");
        return this.api.callMethod("setWebhook", params);
    }
    revokeWebhook() {
        this.log("configuration revoked");
        return this.api.callMethod("deleteWebhook", {});
    }
    async createServer(handler) {
        return new https.Server({
            key: await (0, promises_1.readFile)(this.options.tls.key),
            cert: await (0, promises_1.readFile)(this.options.tls.cert)
        }, async (req) => {
            var e_1, _a;
            if (this.secret && req.headers["X-Telegram-Bot-Api-Secret-Token"] !== this.secret)
                return;
            let chunks = [];
            try {
                for (var req_1 = __asyncValues(req), req_1_1; req_1_1 = await req_1.next(), !req_1_1.done;) {
                    let chunk = req_1_1.value;
                    chunks.push(chunk);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (req_1_1 && !req_1_1.done && (_a = req_1.return)) await _a.call(req_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            let body;
            try {
                body = JSON.parse(Buffer.concat(chunks).toString());
            }
            catch (_) {
                this.log("failed JSON parsing");
                return;
            }
            for (let upd of body.result)
                await handler.handle(upd);
        });
    }
    get secret() {
        var _a, _b;
        return (_b = (_a = this === null || this === void 0 ? void 0 : this.options) === null || _a === void 0 ? void 0 : _a.secret) === null || _b === void 0 ? void 0 : _b.slice(0, 256);
    }
}
exports.Webhook = Webhook;
